;; -*- emacs-lisp -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; $Id: emacs.el,v 1.23 2006-11-06 21:02:42 rscholz Exp $
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Emacs-Einstellungen von Raymond Scholz
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Diese Emacs-Konfiguration ist innerhalb der letzten 7 Jahre von ein
;; paar kmmerlichen Zeilen zu einer ziemlich komplexen Konfiguration
;; gewachsen, die insbesondere viele nicht mit FSF Emacs mitgelieferte
;; Pakete bercksichtigt.
;;
;; Basierend auf der ti3-emacs der Universitt Bremen von Carsten
;; Bormann (cabo), vieler weiterer kleiner Quellen, stundenlanger
;; Lektre der Info-Seiten, Lesen der Newsgroups gnu.emacs.help,
;; gnu.emacs.sources, gnu.emacs.gnus, und der Inspiration vieler
;; Menschen (besonderer Dank hier an Dischi, Crunchy, Hans-Jrgen von
;; der Uni Bremen, Kai Gnusjohann^WGrojohann und ShengHou ZHU) ist
;; dieses Sammelsurium entstanden.
;;
;; Ich nutze Emacs 21.3.50 (aktuelle CVS-Version). D.h. einige
;; Einstellungen werden nur mit dieser neuen Version von Emacs
;; funktionieren.
;;
;; Diese Emacs-Konfiguration ist modular, d.h. die Einstellungen sind
;; nach Funktionen oder Paketen gegliedert in einzelnen, handlichen
;; Dateien abgelegt.
;;
;; Wenn Du gerade die HTML-Version dieser Einstellungen betrachtest,
;; kannst Du mit dem Browser unten die Einstellungen der einzelnen
;; Pakete anklicken.
;;
;; Wenn Du gerade in Emacs eine dieser Dateien betrachtest, findest Du
;; an vielen Stellen einen Verweis der Form:
;;
;;  (Info-goto-node "(Emacs)Copying")
;;
;; Gehe mit dem Cursor hinter die schlieende Klammer und drcke `C-x
;; C-e' - und schon wird die zu der aktuellen Einstellung passende
;; Info-Seite angezeigt.
;;
;; Die offizielle Quelle dieser Dateien ist
;;   <http://www.zonix.de/projects/emacs/config>
;;
;;
;;                                                     Ray, 11.05.03
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; hier liegen alle meine Emacs- und Gnus-Einstellungen
(defvar zonix-elisp-dir "~/el/")

;; Datei laden, wenn vorhanden, sonst Warnung
(defun zonix-load-part (part)
  (let ((fullname (concat zonix-elisp-dir part)))
    (if (file-exists-p fullname)
	(load fullname)
      (message (format "Loading %s (source)...failed" fullname)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Ganz wichtige Dinge...
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(zonix-load-part "emacs_user.el")        ; Benutzer-spezifische Einstellungen
(zonix-load-part "emacs_internals.el")   ; Emacs-Interna
(zonix-load-part "emacs_keys.el")        ; Tastenkrzel
(zonix-load-part "emacs_abbrev.el")      ; Abkrzungen

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Pakete
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Emacs-Funktionserweiterungen

(zonix-load-part "emacs_ibuffer.el")     ; alternative Bufferlist

;; ibuffer.el ist besser...
;; (zonix-load-part "emacs_bs.el")          ; alternative Bufferlist

(zonix-load-part "emacs_faces.el")       ; Darstellung
;(zonix-load-part "emacs_print.el")       ; Druckfunktionen
(zonix-load-part "emacs_unicode.el")     ; Unicode und UTF-8-Untersttzung

;; habe keine Wheel-Maus ;-)
;; (zonix-load-part "emacs_wheel.el")       ; Wheel-Mouse-Untersttzung

;; komplexe Pakete
(zonix-load-part "emacs_speedbar.el")    ; File-Browser
(zonix-load-part "emacs_ecb.el")         ; Emacs Code Browser
(zonix-load-part "emacs_jde.el")         ; Java Development Environment
(zonix-load-part "emacs_gnus.el")        ; Newsreader
(zonix-load-part "emacs_table.el")       ; ASCII-Tabellen bearbeiten
(zonix-load-part "emacs_tramp.el")       ; Remote Dateien bearbeiten
(zonix-load-part "emacs_url.el")         ; URL-Package
(zonix-load-part "emacs_wiki.el")        ; Wiki

;; kleine, ntzliche Tools
(zonix-load-part "emacs_calculator.el")  ; einfacher Taschenrechner
(zonix-load-part "emacs_calendar.el")    ; Kalendar
(zonix-load-part "emacs_crontab.el")     ; Crontab-Mode
(zonix-load-part "emacs_flyspell.el")    ; Rechtschreibkontrolle
(zonix-load-part "emacs_hideshow.el")    ; Verstecken von Code
;(zonix-load-part "emacs_pclcvs.el")      ; CVS-Versionskontrolle
(zonix-load-part "emacs_w3m.el")         ; w3m-Mode
(zonix-load-part "emacs_web.el")         ; ntzliche Funktionen im WWW
(zonix-load-part "emacs_weird.el")       ; verrckte Sachen

;; Texterzeugung
(zonix-load-part "emacs_letter.el")      ; Serienbriefe mit LaTeX
(zonix-load-part "emacs_tex.el")         ; TeX und LaTeX

;; Modes fr Programmiersprachen etc.
(zonix-load-part "emacs_c.el")           ; C-Mode
(zonix-load-part "emacs_csp.el")         ; CSP-Mode
(zonix-load-part "emacs_html.el")        ; HTML-Dateien
(zonix-load-part "emacs_perl.el")        ; Perl-Mode
(zonix-load-part "emacs_php.el")         ; PHP-Mode
(zonix-load-part "emacs_python.el")      ; Python-Mode
(zonix-load-part "emacs_sawfish.el")     ; Sawfish-Mode
(zonix-load-part "emacs_sgml.el")        ; SGML-Mode


(zonix-load-part "emacs_testing.el")     ; noch zu testende Einstellungen


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Customize-Einstellungen landen hier:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(custom-set-variables
  ;; custom-set-variables was added by Custom.
  ;; If you edit it by hand, you could mess it up, so be careful.
  ;; Your init file should contain only one such instance.
  ;; If there is more than one, they won't work right.
 '(ecb-options-version "2.25"))
(custom-set-faces
  ;; custom-set-faces was added by Custom.
  ;; If you edit it by hand, you could mess it up, so be careful.
  ;; Your init file should contain only one such instance.
  ;; If there is more than one, they won't work right.
 '(emacs-wiki-link-face ((t (:foreground "blue" :underline "blue" :weight bold)))))
